//
//  word-cloud.js
//  Library
//
//  Created by Jonathon Mah on 2013-03-18.
//  Copyright (c) 2013 Delicious Monster Software. All rights reserved.
//


function WordCloud(chartDiv, svg, width, height) {
    this.chartDiv = chartDiv;
    chartDiv.classed("word-cloud", true);
    this.svg = svg;
    this.width = width;
    this.height = height;

    this.dataFunction = function() { return []; };
    this.rotations = [-90, 0, 90];

    this.font = "Impact";
    this.fill = d3.scale.ordinal()
        .range(colorbrewer.Set3[12]);

    this.centeredG =
        svg
        .attr("width", width)
        .attr("height", height)
        .append("g")
        .attr("transform", "translate(" + (width / 2) + "," + (height / 2) + ")");

    this.layout = d3.layout.cloud()
        .size([width, height])
        .timeInterval(10)
        .fontSize(function(d) { return d.size; })
        .padding(1);
}


WordCloud.prototype.update = function() {
    var self = this;
    var rotations = this.rotations;

    this.layout.stop();

    // Capture random
    var random = Math.random;
    this.layout.random = random;

    var terms = this.dataFunction();
    this.layout
        .font(this.font)
        .rotate(function() {
            var index = ~~(random() * rotations.length);
            return rotations[index];
        })
        .words(terms)
        .on("end", function(words) { self.draw(words); })
        .start();
};

WordCloud.prototype.draw = function(words) {
    var clickHandler = this.clickHandler;
    var fill = this.fill;

    var text =
        this.centeredG
        .selectAll("text")
        .data(words, function (d) { return d.text; });

    text
        .transition()
        .duration(750)
        .attr("transform", function(d) { return "translate(" + [d.x, d.y] + ")rotate(" + d.rotate + ")"; })
        .style("font-size", function(d) { return d.size + "px"; });

    text
        .enter()
        .append("text")
        .classed("clickable", !!clickHandler)
        .on("click", function(d) {
            if (clickHandler)
                clickHandler(d.text);
        })
        .style("font-size", function(d) { return d.size + "px"; })
        .style("font-family", this.font)
        .style("fill", function(d, i) { return fill(i); })
        .attr("text-anchor", "middle")
        .attr("transform", function(d) {
            return "translate(" + [d.x, d.y] + ")rotate(" + d.rotate + ")";
        })
        .text(function(d) { return d.text; })
        .style("opacity", 1e-6)
        .transition()
        .delay(250)
        .duration(500)
        .style("opacity", 1);

    var exitGroup = this.centeredG.append("g");
    var exitGroupNode = exitGroup.node();
    text.exit().each(function() {
        exitGroupNode.appendChild(this);
    });
    exitGroup.transition()
        .duration(500)
        .style("opacity", 1e-6)
        .remove();
};
